--[[ ExtraActionButton
     Move the ExtraActionButton1 on ExtraActionBarFrame around
     http://www.wowinterface.com/forums/showthread.php?p=248734#post248734
  ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
}
local misc = {
  ["desc"] = "Move the ExtraActionButton1 on ExtraActionBarFrame around";
  ["GUI"] = {
    [1] = {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    };
    [2] = {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    };
  };
}

local M = FixR:NewModule("ExtraActionButton", DefaultO, misc)

local EABFrame = nil
local EABFrameEvents = {};
local moving = false

function EABFrameEvents:UPDATE_EXTRA_ACTIONBAR(...)
  if HasExtraActionBar() then
    ExtraActionButton1.style:SetTexture(nil)
  end
end

function M:Load()
  if EABFrame == nil then --create Frame if does not already exist
    EABFrame = CreateFrame("Frame", "EABFrame", UIParent)
    EABFrame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
    EABFrame:SetFrameStrata("HIGH")
    EABFrame:SetSize(40, 40)
    
    --blue background
    EABFrame.texture = EABFrame:CreateTexture()
    EABFrame.texture:SetAllPoints(EABFrame)
    if FixR:IsLeg() then
      EABFrame.texture:SetColorTexture(0.67, 0.67, 1, 0.01)
    else
      EABFrame.texture:SetTexture(0.67, 0.67, 1, 0.01)
    end
    
    EABFrame:SetScript("OnEvent", function(self, event, ...)
      EABFrameEvents[event](self, ...); -- call one of the functions above
    end);
    EABFrame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
    EABFrame:SetScript("OnDragStop", function(self)
      self:StopMovingOrSizing()
      local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
      M.O["framePoint"] = point or "CENTER"
      M.O["frameRelativeTo"] = relativeTo or "UIParent"
      M.O["frameRelativePoint"] = relativePoint or "CENTER"
      M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
      M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
    end);
    
    --[[
    EABFrame:EnableMouse(true)
    EABFrame:SetScript("OnMouseUp", function(self, button, ...)
      if button == "MiddleButton" then
        --self:SetSize(20, 20) -- protected
        --self:Hide() -- protected
        --self:EnableMouse(false) -- protected
        --self:SetFrameStrata("BACKGROUND") -- protected
        --/run EABFrame:SetFrameStrata("HIGH")
      end
    end);
    --]]
    
  end
  
  for k, v in pairs(EABFrameEvents) do
    EABFrame:RegisterEvent(k); -- Register all events for which handlers have been defined
  end
  
  local f = ExtraActionBarFrame
  f:SetParent(EABFrame)
  f:ClearAllPoints()
  f:SetPoint("CENTER", 0, 0)
  f.ignoreFramePositionManager = true
  
  ExtraActionButton1.style:SetTexture(nil)
  ExtraActionButton1:SetSize(40, 40) --original button size: 52x52
  
  EABFrame:Show()
  
end
function M:Unload()
  if EABFrame then
    for k, v in pairs(EABFrameEvents) do
      EABFrame:UnregisterEvent(k); -- Unregister all events for which handlers have been defined
    end
    EABFrame:Hide()
  end
end

function M:SetOption(o)
  if o == "move" then
    if moving then
      moving = false
      if FixR:IsLeg() then
        EABFrame.texture:SetColorTexture(0.67, 0.67, 1, 0.01) -- cannot :Hide() in combat, so make it nearly transparent
      else
        EABFrame.texture:SetTexture(0.67, 0.67, 1, 0.01) -- cannot :Hide() in combat, so make it nearly transparent
      end
      EABFrame:SetMovable(false) --click-through
      EABFrame:EnableMouse(false)
      EABFrame:RegisterForDrag("")
    else
      moving = true
      if FixR:IsLeg() then
        EABFrame.texture:SetColorTexture(0.67, 0.67, 1, 1) --opaque
      else
        EABFrame.texture:SetTexture(0.67, 0.67, 1, 1) --opaque
      end
      EABFrame:SetMovable(true);
      EABFrame:EnableMouse(true);
      EABFrame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("ExtraActionButton: now "..(moving and "moving" or "fixed"))
  elseif o == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    EABFrame:ClearAllPoints()
    EABFrame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  end
end
function M:ShowOptions()
  FixR:Echo("  ExtraActionButton move: toggle moving the frame")
  FixR:Echo("  ExtraActionButton reset: reset the frame's position")
end